#include "checker.h"

Checker::Checker( GlyphID id, OldGlyphBg* m_transBg, TransWindow* m_tw )
{
	model = GlyphDB::globalDB()->getGlyphByID( id );
	transBg = m_transBg;
	tw = m_tw;
}

Checker::~Checker()
{
}

double Checker::vzd( Point a, Point b )
{
	return (a.x-b.x)*(a.x-b.x) + (a.y-b.y)*(a.y-b.y);
}

double Checker::checkShakiness( Glyph* scrib )
{
	return 0.0;
}

LineList Checker::fitX( LineList ll ) 
{
	//prvotny posun na xovej osi
	LineList res;
	double shiftx = model->lines[0][0].x - ll[0][0].x;
	for (int l=0; l<(int)ll.size(); l++) {
		Line tmpline;
		for (int p=0; p<(int)ll[l].size(); p++) {
			Point tmppoint;
			tmppoint = ll[l][p];
			tmppoint.x=ll[l][p].x+shiftx;
			tmpline.push_back(tmppoint);
		}
		res.push_back(tmpline);
	}
	return res;
}


LineList Checker::fitXY( LineList ll ) 
{
	//posun na xovej a yovej osi
	LineList res;
	for (int l=0; l<min((int)(model->lines.size()),(int)ll.size()); l++) {
		double shiftx = model->lines[l][0].x - ll[l][0].x;
		double shifty = model->lines[l][0].y - ll[l][0].y;
		if ( POSUN == 1 ) {
			double sminxp=0, sminyp=0, smaxxp=0, smaxyp=0, mminxp=0, mminyp=0, mmaxxp=0, mmaxyp=0; //indexy extremov
			for (int p=0; p<(int)ll[l].size(); p++) {
				if (ll[l][p].x<ll[l][sminxp].x) { sminxp=p; }
				if (ll[l][p].x>ll[l][smaxxp].x) { smaxxp=p; }
				if (ll[l][p].y<ll[l][sminyp].y) { sminyp=p; }
				if (ll[l][p].y>ll[l][smaxyp].y) { smaxyp=p; }
			}
			for (int p=0; p<(int)model->lines[l].size(); p++) {
				if (model->lines[l][p].x<model->lines[l][mminxp].x) {mminxp=p; }
				if (model->lines[l][p].x>model->lines[l][mmaxxp].x) {mmaxxp=p; }
				if (model->lines[l][p].y<model->lines[l][mminyp].y) {mminyp=p; }
				if (model->lines[l][p].y>model->lines[l][mmaxyp].y) {mmaxyp=p; }
			}
			shiftx = model->lines[l][mminxp].x - ll[l][sminxp].x;
			shifty = model->lines[l][mminyp].y - ll[l][sminyp].y;
		}
		Line tmpline;
		for (int p=0; p<(int)ll[l].size(); p++) {
			Point tmppoint;
			tmppoint = ll[l][p];
			tmppoint.x=ll[l][p].x+shiftx;
			tmppoint.y=ll[l][p].y+shifty;
			tmpline.push_back(tmppoint);
		}
		res.push_back(tmpline);
	}
	return res;
}

Line Checker::skew( Line line, double alpha ) {
	//zosikmenie = zosikmenie y-ovej osi pri zachovanej vyske
	Line res;
	if (abs(alpha)<0.0005) return line;
	REP(p, line.size()) {
		Point tmppoint;
		tmppoint = line[p];
		tmppoint.x += line[p].y*tan(alpha);
		res.push_back(tmppoint);
	}
	return res;
}

double Checker::dtw( Line mod, Line scr) {
	vector < vector <double> > D;
	vector <double> tmp;
	tmp.push_back( vzd( mod[0], scr[0]) );
	for (int i=1; i<(int)scr.size(); i++) {
		tmp.push_back( vzd( mod[0], scr[i] ) + tmp[i-1]);
	}
	D.push_back(tmp);
	for (int t=1; t<(int)mod.size(); t++) {
		tmp.clear();
		for (int i=0; i<(int)scr.size(); i++) {
			if (i==0) {
				tmp.push_back( vzd( mod[t], scr[i] ) + D[t-1][i]);
			} else {
				if (D[t-1][i-1]<D[t-1][i] && D[t-1][i-1]<tmp[i-1]) {
					tmp.push_back( vzd( mod[t], scr[i] ) + D[t-1][i-1]);
				} else if (D[t-1][i]<D[t-1][i-1] && D[t-1][i]<tmp[i-1]) {
					tmp.push_back( vzd( mod[t], scr[i] ) + D[t-1][i]);
				} else {
					tmp.push_back( vzd( mod[t], scr[i] ) + tmp[i-1]);
				}
			}
		}
		D.push_back(tmp);
	}
	return D.back().back();
}


LineList Checker::fitInclination( LineList ll ) 
{
	//sklon pisma
	LineList res;
	for (int l=0; l<min((int)ll.size(), (int)(model->lines.size())); l++) {
		double mindtw=0.0;
		double minal=-100;
		for (double al=-M_PI/4; al<0; al+=0.05) {
			Line tmpline=skew(ll[l], al);
			double dtwr = dtw( model->lines[l], tmpline );
			if (minal==-100 || dtwr<mindtw) {
				mindtw = dtwr;
				minal = al;
			}
		}
		for (double al=0; al<M_PI/4; al+=0.05) {
			Line tmpline=skew(ll[l], al);
			double dtwr = dtw( model->lines[l], tmpline );
			if (dtwr<mindtw) {
				mindtw = dtwr;
				minal = al;
			}
		}
		res.push_back( skew( ll[l], minal ) );
		if (minal<-0.05) tw->addText( format_string("%d. line -- small inclination\n", l+1) );
		if (minal>0.05) tw->addText( format_string("%d. line -- big inclination\n", l+1) );
	}
	return res;
}

LineList Checker::fitWidthHeight( LineList ll ) 
{
	//zmena sirky a vysky
	LineList res;
	for (int l=0; l<min((int)ll.size(), (int)(model->lines.size())); l++) {
		double sminxp=0, sminyp=0, smaxxp=0, smaxyp=0, mminxp=0, mminyp=0, mmaxxp=0, mmaxyp=0; //indexy extremov
		for (int p=0; p<(int)ll[l].size(); p++) {
			if (ll[l][p].x<ll[l][sminxp].x) { sminxp=p; }
			if (ll[l][p].x>ll[l][smaxxp].x) { smaxxp=p; }
			if (ll[l][p].y<ll[l][sminyp].y) { sminyp=p; }
			if (ll[l][p].y>ll[l][smaxyp].y) { smaxyp=p; }
		}
		for (int p=0; p<(int)model->lines[l].size(); p++) {
			if (model->lines[l][p].x<model->lines[l][mminxp].x) {mminxp=p; }
			if (model->lines[l][p].x>model->lines[l][mmaxxp].x) {mmaxxp=p; }
			if (model->lines[l][p].y<model->lines[l][mminyp].y) {mminyp=p; }
			if (model->lines[l][p].y>model->lines[l][mmaxyp].y) {mmaxyp=p; }
		}
		double swidth = ll[l][smaxxp].x - ll[l][sminxp].x;
		double mwidth = model->lines[l][mmaxxp].x - model->lines[l][mminxp].x;
		double sheight = ll[l][smaxyp].y - ll[l][sminyp].y;
		double mheight = model->lines[l][mmaxyp].y - model->lines[l][mminyp].y;
		double indexx;
		if ( abs( swidth ) < EPS )
			indexx = mwidth;
		else 
			indexx = mwidth/swidth;
		double indexy;
		if ( abs( sheight ) < EPS )
			indexy = mheight;
		else 
			indexy = mheight/sheight;
		double beginx = ll[l][sminxp].x;
		double beginy = ll[l][sminyp].y;
		if (ll[l][smaxyp].y-model->lines[l][mmaxyp].y>3.0) tw->addText( format_string("%d. line -- much under the baseline\n", l+1 ) );
		if (ll[l][smaxyp].y-model->lines[l][mmaxyp].y<-4.0) tw->addText( format_string("%d. line -- much above the baseline\n", l+1 ) );
		if (swidth-mwidth>8.0) tw->addText( format_string("%d. line -- too wide\n", l+1) );
		if (swidth-mwidth<-2.0) tw->addText( format_string("%d. line -- too narrow\n", l+1) );
		if (sheight-mheight>8.0) tw->addText( format_string("%d. line -- too high\n", l+1) );
		if (sheight-mheight<-3.0) tw->addText( format_string("%d. line -- too small\n", l+1) );
		Line tmpline;
		for (int p=0; p<(int)ll[l].size(); p++) {
			Point tmppoint;
			tmppoint = ll[l][p];
			tmppoint.x=(ll[l][p].x-beginx)*indexx+beginx;
			tmppoint.y=(ll[l][p].y-beginy)*indexy+beginy;
			tmpline.push_back(tmppoint);
		}
		res.push_back(tmpline);
	}
	return res;
}

LineList Checker::fitScribble( Glyph* scrib )
{
	LineList res = scrib->lines;
	res = fitX( res );
	transBg->addLineList( lineList3( res ), BLACK );
	res = fitWidthHeight( res );
	res = fitXY( res );
	res = fitInclination( res );
	transBg->addLineList( lineList3( res ), BLUE );
//	transBg->addLineListWithCircle( lineList3( res ), BLUE, 0, 5 );
	return res;
}

double Checker::checkAuth( Glyph* scrib )
{
	double res=0.0;
	if (model->lines.size()!=scrib->lines.size()) tw->addText( "Wrong number of lines\n" );
	LineList fitted = fitScribble( scrib );
	for (int l=0; l<(int)min(model->lines.size(), fitted.size()); l++) {
		vector < vector <double> > D;
		vector <double> tmp;
		vector < vector <pair <int, int> > > fi;
		vector <pair <int, int> > tmpfi;
		tmp.push_back(vzd(model->lines[l][0], fitted[l][0]));
		tmpfi.push_back(pair<int, int>(-1,-1));
		for (int i=1; i<(int)fitted[l].size(); i++) {
			tmp.push_back(vzd(model->lines[l][0],fitted[l][i])+tmp[i-1]);
			tmpfi.push_back(pair<int,int>(0,i-1));
		}
		D.push_back(tmp);
		fi.push_back(tmpfi);
		for (int t=1; t<(int)model->lines[l].size(); t++) {
			tmp.clear();
			tmpfi.clear();
			for (int i=0; i<(int)fitted[l].size(); i++) {
				if (i==0) {
					tmp.push_back(vzd(model->lines[l][t],fitted[l][i])+D[t-1][i]);
					tmpfi.push_back(pair<int,int>(t-1,i));
				} else {
					if (D[t-1][i-1]<D[t-1][i] && D[t-1][i-1]<tmp[i-1]) {
						tmp.push_back(vzd(model->lines[l][t],fitted[l][i])+ D[t-1][i-1]);
						tmpfi.push_back(pair<int,int>(t-1,i-1));
					} else if (D[t-1][i]<D[t-1][i-1] && D[t-1][i]<tmp[i-1]) {
						tmp.push_back(vzd(model->lines[l][t],fitted[l][i])+ D[t-1][i]);
						tmpfi.push_back(pair<int,int>(t-1,i));
					} else {
						tmp.push_back(vzd(model->lines[l][t],fitted[l][i])+ tmp[i-1]);
						tmpfi.push_back(pair<int,int>(t,i-1));
					}
//					tmp.push_back(vzd(model->lines[l][t],fitted[l][i])+ min(min(D[t-1][i-1], D[t-1][i]),tmp[i-1]));
				}
			}
			D.push_back(tmp);
			fi.push_back(tmpfi);
		}
		res+=D.back().back();
		vector <pair <int,int> > Z;
		Z.push_back(pair<int,int>(fi.back().back().first, fi.back().back().second));
		int k=0;
		int mo=Z[k].first, sc=Z[k].second;
		while (mo>0 || sc>0) {
			k++;
			Z.push_back(fi[mo][sc]);
			mo=Z[k].first;
			sc=Z[k].second;
		}
		reverse(Z.begin(), Z.end());
		/*FILE * f;
		if (f = fopen("data/log.txt", "w")) {
			REP(i, Z.size()) {
				fprintf(f, "%.2lf ", D[Z[i].first][Z[i].second]);
				fprintf(f, "\n");
			}
			fclose(f);
		} else {
			printf("unable to open log file\n");
		} */
		if (Z.size()>10) {
			FOR(i, 10, Z.size()) {
				double diff = D[Z[i].first][Z[i].second] - D[Z[i-10].first][Z[i-10].second];
				if (diff>150) {
					//tu je chyba, treba zakruzkovat
					transBg->addCircle(l, Z[i-10].second);
					break;
				}
			}
		}
	}
	return res;
}

