
#include "glyphanimbackground.h"
#include "mydrawingarea.h"
#include "glyphdb.h"

#define ANIM_DELAY 20

GlyphAnimBackground::GlyphAnimBackground( GlyphID id, MyDrawingArea* dArea )
	: dArea(dArea), line(0), point(0)
{
	lines = GlyphDB::globalDB()->getLinesByID(id);
	restart();
}

GlyphAnimBackground::~GlyphAnimBackground() {
	timer.disconnect();
}

void GlyphAnimBackground::restart()
{
	timer.disconnect();
	line = 0; point = 0;
	timer = Glib::signal_timeout().connect( sigc::mem_fun( *this, &GlyphAnimBackground::on_timer ), ANIM_DELAY );
}

bool GlyphAnimBackground::onExposeEvent( GdkEventExpose *event, Cairo::RefPtr<Cairo::Context> context, MyDrawingArea *dArea  ) {

	context->set_line_width(2);
	context->set_source_rgba( GREEN );

	//vykreslenie glyphu
	if (lines.size()>0) {
		context->translate( 0, dArea->getBaseLineY() );
		context->set_line_width(3);
		for(int l=0; l<(int)lines.size() && l<=line; ++l) {
			if (lines[l].size()>1) {
				context->move_to(lines[l][0].x, lines[l][0].y );
				for(int p=1; p<(int)lines[l].size() && (l<line || p<=point); ++p) {
					context->line_to(lines[l][p].x, lines[l][p].y );
				}
				context->stroke();
			}
			if (lines[l].size()==1) {
				//nakresli bodku ;)
				Line &ll = lines[l];
				context->arc(ll[0].x, ll[0].y, 0.8, 0.0, 2.0 * M_PI);
				context->set_line_width(0);
				context->set_source_rgba( GREEN );    // partially translucent
				context->fill();
			}
		}
	} // lines.size()>0
	
	if ((line!=0 || point!=0) && line<(int)lines.size() && point<(int)lines[line].size() ) {
		//gulicka na mieste kde prave animujeme (kvoli viacnasobnym ciaram atd
				Line &ll = lines[line];
				context->arc(ll[point].x, ll[point].y, 3.0, 0.0, 2.0 * M_PI);
				context->set_line_width(0);
				context->set_source_rgba( 0.9, 0.2, 0.2, 1.0  );
				context->fill();
	}

	context->stroke();
	return true;
}

void GlyphAnimBackground::drawLast()
{
	dArea->queue_draw();
}

bool GlyphAnimBackground::on_timer()
{
	++point;
	if (line<(int)lines.size() && point>(int)lines[0].size()) {
		++line;
		point=0;
	}
	if (line<(int)lines.size()) {
		drawLast();
		return true;
	} else {
		line=0; point=0;
		drawLast();
		return false;
	}
}


