
#include <gtkmm/label.h>
#include <cairomm/context.h>

#include "config.h"
#include "mydrawingarea.h"



MyDrawingArea::MyDrawingArea( bool capture )
	:
		lineHeight( DEFAULT_GLYPH_HEIGHT ),
		capturing( false )
{
	this->capture = capture;
	set_size_request( DEFAULT_DAREA_WIDTH, DEFAULT_DAREA_HEIGHT );
	add(m_dArea);
	m_dArea.signal_expose_event().connect(sigc::mem_fun(*this,&MyDrawingArea::on_dArea_expose_notify_event));
	baseLineY = (DEFAULT_DAREA_HEIGHT - lineHeight) / 2 + lineHeight;
}

bool MyDrawingArea::on_motion_notify_event( GdkEventMotion *event ) {
	if (capture && capturing)
		addPoint( event->x, event->y );
	return true;
}

bool MyDrawingArea::on_button_press_event( GdkEventButton *event ) {
	lineList.push_back( Line() );
	capturing = true;
	if (capture && capturing)
		addPoint( event->x, event->y );
	return true;
}


bool MyDrawingArea::on_button_release_event( GdkEventButton *event ) {
	capturing = false;
	return true;
}


bool MyDrawingArea::on_dArea_expose_notify_event( GdkEventExpose *event ) {
	return dArea_paint( event, false );
}

/**
 * prekresli co treba:
 * event != NULL -> prekresli vsetko v regione danom v event
 * event == NULL && clear == false -> iba dokresli posledny bod ciary
 * event == NULL && clear == true -> prekresli cele ;)
 */
bool MyDrawingArea::dArea_paint( GdkEventExpose *event, bool clear ) {
	Glib::RefPtr<Gdk::Window> window = m_dArea.get_window();
	if(window)
	{
//	printf( "Expose event\n" );

		Gtk::Allocation allocation = m_dArea.get_allocation();
		const int width = allocation.get_width();
		const int height = allocation.get_height();
//		int lesser = MIN(width, height);

		// just some stuff to see something over the map
		int xc, yc;
		xc = width / 2;
		yc = height / 2;

		Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();

		// clip to the area indicated by the expose event so that we only redraw
		// the portion of the window that needs to be redrawn
		if (event) {
			cr->rectangle(event->area.x, event->area.y,
				event->area.width, event->area.height);
			cr->clip();
		}

		// jednotne pozadie
		if (event || clear) {
			cr->save();
			cr->rectangle( 0, 0, width, height );
			cr->set_source_rgba( 1.0, 1.0, 1.0, 1.0 );
			cr->fill();
			cr->restore();
		}
		// vykreslime pozadie/ia
		if (event || clear)
			for( int i=0; i<(int)bgList.size(); ++i ) {
				cr->save();
				bgList[i]->onExposeEvent( event, cr, this );
				cr->restore();
			}

		// mame vobec nejaku ciaru?
		if (lineList.size()>0) {
			cr->translate( 0, baseLineY );
			cr->set_line_width(1);
			if ( event || clear ) {
				for(int l=0; l<(int)lineList.size(); ++l) {
					if (lineList[l].size()>1) {
						cr->move_to(lineList[l][0].x, lineList[l][0].y );
						for(int p=1; p<(int)lineList[l].size(); ++p) {
							cr->line_to(lineList[l][p].x, lineList[l][p].y );
						}
						cr->stroke();
					}
					if (lineList[l].size()==1) {
						//nakresli bodku ;)
						Line &ll = lineList[l];
						cr->arc(ll[0].x, ll[0].y, 0.8, 0.0, 2.0 * M_PI);
						cr->set_line_width(0);
						cr->set_source_rgba(.0, .0, .0, 1);    // partially translucent
						cr->fill();
					}
				}
			} else if (lineList.back().size()>1) {
				//chceme iba posledny bod
				Line &ll = lineList.back();
				cr->move_to(ll[ll.size()-2].x, ll[ll.size()-2].y);
				cr->line_to(ll[ll.size()-1].x, ll[ll.size()-1].y);
				cr->stroke();
			} else if (lineList.back().size()==1) {
				// nakresli bodku ;)
				Line &ll = lineList.back();
				cr->arc(ll[0].x, ll[0].y, 0.8, 0.0, 2.0 * M_PI);
				cr->set_line_width(0);
				cr->set_source_rgba(.0, .0, .0, 1);    // partially translucent
				cr->fill();

			}
		} // lineList.size()>0
	}

	return false;
}

void MyDrawingArea::on_size_allocate( Gtk::Allocation &allocation ) {
	Gtk::EventBox::on_size_allocate( allocation );
	baseLineY = (m_dArea.get_allocation().get_height() - lineHeight) / 2 + lineHeight;
}

bool MyDrawingArea::addPoint( gdouble x, gdouble y ) {
	Point p;
	p.x = x;
	p.y = y - baseLineY;
//	printf( "adding point %lf %lf\n", p.x, p.y );
	lineList.back().push_back(p);
	// prekreslime (inac by sa prekreslilo iba az ked by sme mu nejako vyrobili expose event...
	// druhy parameter hovori ze prekresli iba poslednu ciarocku
	dArea_paint( NULL, false );
	return true;
}

bool MyDrawingArea::startCapture() {
	capture = true;
	lineList.clear();
	dArea_paint( NULL, true );
	return 1;
}

LineList MyDrawingArea::stopCapture() {
	capture = false;
	capturing = false;
	return lineList;
}

LineList MyDrawingArea::getCapture() {
	return lineList;
}

bool MyDrawingArea::addBg( Background * bg ) {
	bool contain = false;
	for (int i=0; i<(int)bgList.size(); i++) {
		if ( bgList[i] == bg ) contain = true;
	}
	if ( !contain ) bgList.push_back(bg);
	return 1;
}

bool MyDrawingArea::removeBg( Background * bg ) {
	for (int i=0; i<(int)bgList.size(); i++) {
		if ( bgList[i] == bg )
			bgList.erase(bgList.begin()+i);
	}
	return 1;
}

bool MyDrawingArea::clearBg() {
	bgList.clear();
	return 1;
}

bool MyDrawingArea::clearCapture() {
	stopCapture();
	startCapture();
	return true;
}

bool MyDrawingArea::clearButDrawCapture() {
	stopCapture();
	capture = true;
	lineList.clear();
	return true;
}

Gtk::DrawingArea* MyDrawingArea::get_dArea() {
	return &m_dArea;
}


int MyDrawingArea::getBaseLineY() {
	return baseLineY;
}

int MyDrawingArea::getLineHeight() {
	return lineHeight;
}

void MyDrawingArea::setLineHeight( int height ) {
	lineHeight = height;
	// TODO preratat uz capturnute ciary
}

void MyDrawingArea::setSize( int w, int h) {
	set_size_request( w, h );
}

