#include "scribblertask.h"

ScribblerTask::ScribblerTask() 
	:
//		bg( 100, 80 ),
		glyphBg( NULL ),
		oldGlyphs( NULL ),
		checker( NULL ),
		buttonNew( "_Nové písmeno" ),
		buttonProp( "_Zisti vlastnosti" ),
		buttonAnim( "_Animuj" ),
		buttonShowProp( "_Ukáž vlastnosti" )
{
	id = "Písanka s celými písmenami";
	editingArea.addBg( &lineBg );
	actualID = GlyphDB::globalDB()->getRandomID();
	glyphBg = new GlyphWholeBackground( actualID );
	glyphAnimBg = new GlyphAnimBackground( actualID, &editingArea );
	oldGlyphs = new OldGlyphBg();
	transGlyphBg = new OldGlyphBg();
	editingArea.addBg( glyphBg );
	editingArea.addBg( oldGlyphs );
	editingArea.addBg( glyphAnimBg );
	tw.clear();
	tw.addBg( &lineBg );
	tw.addBg( transGlyphBg );

	buttonNew.set_use_underline( true );
	buttonProp.set_use_underline( true );
	buttonShowProp.set_use_underline( true );
	buttonAnim.set_use_underline( true );
	buttonNew.signal_clicked().connect(sigc::mem_fun(*this,&ScribblerTask::on_buttonNew_clicked));
	buttonProp.signal_clicked().connect(sigc::mem_fun(*this,&ScribblerTask::on_buttonProp_clicked));
	buttonAnim.signal_clicked().connect(sigc::mem_fun(*this,&ScribblerTask::on_buttonAnim_clicked));
	buttonShowProp.signal_clicked().connect(sigc::mem_fun( tw, &TransWindow::show));
	editorButtons.pack_start( buttonProp, false, false, 10 );
	editorButtons.pack_start( buttonShowProp, false, false, 10 );
	editorButtons.pack_start( buttonAnim, false, false, 10 );
	editorButtons.pack_end( buttonNew, false, false, 10 );
	editorWindow.pack_start( editingArea, true, true );
	editorWindow.pack_start( editorButtons, false, false, 10 );
	on_buttonNew_clicked();
}

ScribblerTask::~ScribblerTask() {
	if (glyphBg) delete glyphBg;
	if (checker) delete checker;
	if (oldGlyphs) delete oldGlyphs;
	if (transGlyphBg) delete transGlyphBg;
}

bool ScribblerTask::restart() {
	editingArea.clearCapture();
	return true;
}

Gtk::Widget& ScribblerTask::getMainWidget() {
	return editorWindow;
}

void ScribblerTask::on_buttonNew_clicked(){
        printf("Clearing capture and generating new glyph.\n");
		tw.hide();
		if (transGlyphBg) transGlyphBg->clear();
		editingArea.removeBg( glyphBg );
		editingArea.removeBg( glyphAnimBg );
		if (glyphBg) {
			delete glyphBg;
			glyphBg = NULL;
		}
		if (glyphAnimBg) {
			delete glyphAnimBg;
			glyphAnimBg = NULL;
		}
		if (checker) {
			delete checker;
			checker = NULL;
		}
		if (oldGlyphs) oldGlyphs->clear();
		actualID = GlyphDB::globalDB()->getRandomID();
		glyphBg = new GlyphWholeBackground( actualID );
		glyphAnimBg = new GlyphAnimBackground( actualID, &editingArea );
		editingArea.addBg( glyphBg );
		editingArea.addBg( glyphAnimBg );
        editingArea.clearCapture();
}

void ScribblerTask::on_buttonProp_clicked()
{
	transGlyphBg->clear();
	tw.clearText();
	transGlyphBg->addLineList( lineList3( GlyphDB::globalDB()->getLinesByID( actualID ) ), GREY );
	printf("Checking properties.\n");
	g = new Glyph();
	g->lines=editingArea.getCapture();
	if (g->lines.size()==0) {
		printf("Nothing to be checked, write something.\n");
	} else {
		tw.show();
		oldGlyphs->addLineList( g->lines, GREEN );
//		transGlyphBg->addLineList( lineList3( g->lines ), 0.1, 0.9, 0.1, 0.8 );
		if (checker) {
			delete checker;
			checker = NULL;
		}
		checker = new Checker( actualID, transGlyphBg, &tw );
//		printf("Shakiness = %lf\n", checker->checkShakiness(g));
		int wholesize = 0;
		double auth = checker->checkAuth(g);
		printf("Verification = %lf\n", auth);
		REP(i, g->lines.size()) wholesize+=(int)(g->lines[i].size());
		if (auth>wholesize*10) tw.addText( "Wrong glyph shape" );
		editingArea.clearCapture();
	}
	tw.queue_draw();
}

void ScribblerTask::on_buttonAnim_clicked()
{
	glyphAnimBg->restart();
}


